/*******************************************************************************


  TETRATIMER

  Temporizador de 4 canais programveis individualmente.

  60 minutos mximo por canal.

  Autor: Claudio Larios

  Data: 23/12/2016

  Uso didtico

  Este arquivo  parte integrante do blog Lrios.Tecnologia.ws



******************************************************************************/
#include <16F628A.h>
#use delay(clock=4000000)
#fuses NOWDT,XT, NOPUT, NOPROTECT, BROWNOUT, MCLR, NOLVP, NOCPD//configurao
#zero_ram//zera no nicio
#use fast_io(a)//modo rpido
#use fast_io(b)//modo rpido
//bytes
#byte trisb = 0x86
#byte trisa = 0x85
#byte tmr0  = 0x01
#byte portb = 0x06
#byte porta = 0x05
#byte tmr1h = 0x0f
#byte tmr1l = 0x0e
#byte t1con = 0x10

// lcd
#bit   en     = 0x06.2 //rb2 lcd 'enable' pino 8
#bit   rs     = 0x06.3 //rb3  lcd 'rs' - comando/dado pino 9
#bit   db4    = 0x05.0 //ra0  db4 pino 17
#bit   db5    = 0x05.1 //ra1  db5 pino 18
#bit   db6    = 0x05.2 //ra2  db6 pino 1
#bit   db7    = 0x05.3 //ra3  db7 pino 2
#bit   tris_en     = 0x86.2 //rb2  lcd 'enable' pino 8
#bit   tris_rs     = 0x86.3 //rb3  lcd 'rs' - comando/dado pino 9
#bit   tris_db4    = 0x85.0 //ra0  dir db4 pino 17
#bit   tris_db5    = 0x85.1 //ra1  dir db5 pino 18
#bit   tris_db6    = 0x85.2 //ra2  dir db6 pino 1
#bit   tris_db7    = 0x85.3 //ra3  dir db7 pino 2

//pinos de sadas
#bit   releA  = 0x06.7 //rb7  pino 13
#bit   releB  = 0x06.6 //rb6  pino 13
#bit   releC  = 0x06.5 //rb5  pino 11
#bit   releD  = 0x06.4 //rb4  pino 10
#bit   somA   = 0x06.0 //rb0  pino  6
#bit   somB   = 0x06.1 //rb1  pino  7
//pinos de entrada
#bit   in_b     = 0x05.4 //entrada para teclas pino 3

//bits
#bit int_tmr0_on  = 0x0b.5 //flag on/off de interrupao do timer 0
#bit int_tmr1_on  = 0x10.0 //flag on/off do timer 1
#priority timer1,timer0



//variveis globais
unsigned int8 cnt, minutosA, segundosA, minutosB, segundosB, minutosC, segundosC,minutosD, segundosD;
unsigned int8 selecao,timep;
unsigned int1 f_bipar,f_pauseA,f_pauseB,f_pauseC,f_pauseD;
unsigned int1 f_tr1;
unsigned int1 f_piscaA,f_piscaB,f_piscaC,f_piscaD;
unsigned int8 processo=0;
unsigned int8 cont6=0;
unsigned int8 stpisca=0;

//DEFINES
#define TEMPO_BEEP  2  //durao do beep
#define TEMPO_PAUSA  5  //durao do intervalo entre bipes
#define TEMPO_PISCANDO 100 // periodo piscando aps acionar boto sel

 //=============================================================================
 //    ROTINA DE INTERRUPO DO TIMER 0  (CADA 8MS)
 //=============================================================================
#int_timer0
void interrupt_tmr0()
{
   TMR0 = get_timer0()+131; //carrega TMR0 com o valor
    if(++cnt>=125){ //8ms * 125 = 1s
         cnt=0;

       if(f_pauseA){
         if(--segundosA==0xff){
           segundosA=59;
           if( minutosA>0) --minutosA;
           }
         if(!minutosA&&!segundosA) {
               releA=0;//desliga o rele de sada
               f_bipar=1;//sinaliza para bipar (fim da contagem)
               f_pauseA=0;
         }
       }

        if(f_pauseB){
         if(--segundosB==0xff){
           segundosB=59;
           if( minutosB>0) --minutosB;
           }
         if(!minutosB&&!segundosB) {
               releB=0;//desliga o rele de sada
               f_bipar=1;//sinaliza para bipar (fim da contagem)
               f_pauseB=0;
         }
       }

        if(f_pauseC){
         if(--segundosC==0xff){
           segundosC=59;
           if( minutosC>0) --minutosC;
           }
         if(!minutosC&&!segundosC) {
               releC=0;//desliga o rele de sada
               f_bipar=1;//sinaliza para bipar (fim da contagem)
               f_pauseC=0;
         }
       }
        if(f_pauseD){
         if(--segundosD==0xff){
           segundosD=59;
           if( minutosD>0) --minutosD;
           }
         if(!minutosD&&!segundosD) {
               releD=0;//desliga o rele de sada
               f_bipar=1;//sinaliza para bipar (fim da contagem)
               f_pauseD=0;
         }
       }

    }
}
 //=============================================================================
 //    ROTINA DE INTERRUPO DO TIMER 1 (GERAR SOM NO PIEZO)
 //=============================================================================
#int_timer1
void interrupt_tmr1(){
//nota l 588hz
    tmr1h=0xfc;
    tmr1l=0xae;
    somB=somA;
    somA=!somA;
}

//==============================================================================
//                   ROTINAS PARA LCD
//==============================================================================
//clock para lcd
void lcd_clk(void){
 en=1;
 delay_us(10);//
 en=0;

 }

//envia nible para lcd
 void env_nib(int8 b){

 if(b&0x10)db4=1; else db4=0;
 if(b&0x20)db5=1; else db5=0;
 if(b&0x40)db6=1; else db6=0;
 if(b&0x80)db7=1; else db7=0;
 lcd_clk();
 }

 //envia comando para lcd
 void env_lcd_c(int8 val){
 rs=0;
 env_nib(val);
 env_nib(swap(val));
 delay_ms(10);
}

//envia dados para lcd
 void env_lcd_d(int8 val){
 rs=1;
 env_nib(val);
 env_nib(swap(val));
 }

//inicializao do lcd
void lcd_init(void){
 int8 a;

  rs=0;
  tris_en=0;//direo dos pinos como saida
  tris_rs=0;
  tris_db4=0;
  tris_db5=0;
  tris_db6=0;
  tris_db7=0;
  db4=0;//situao inicial resetados
  db5=0;
  db6=0;
  db7=0;
  en=0;
  delay_ms(50);//delay inicial
  db4=1; db5=1;// setados
  for(a=0;a<3;a++) {lcd_clk(); delay_ms(5);}// envia 3,3,3
  db4=0;//envia 2
  lcd_clk();//pulso em 'en'
  delay_ms(5);
  env_lcd_c(0x28);//ajusta para 4 bits o modo de envio
  env_lcd_c(0x06);// no shift
  env_lcd_c(0x0c);//Display ON, Cursor OFF, Blink OFF
  env_lcd_c(0x01);//limpa tela
  delay_ms(10);

}
//-------------------------FIM DAS ROTINAS DE LCD--------------------------------

 //=============================================================================
 //    ROTINA DE CONVERSO HEXA PARA DECIMAL
 //=============================================================================
unsigned int8 Conv_Dec(unsigned int8 valor){

  unsigned int8 a=0;
  unsigned int8 d=0;
   for(a=0;valor>9;a++){
     valor-=10;
     d++;
   }
   return(d<<4|valor);

}

 //=============================================================================
 //    ROTINA DE LEITURA DOS BOTES DE COMANDO
 //=============================================================================
unsigned int8 le_botao(){

     unsigned int8 res=0;
     disable_interrupts(int_timer0);//desabilita para testar os botes
     porta&=0b11110000;//desliga db7...db4
     porta|=0x08;//testa segundos
     delay_us(100);
     if(in_b)res|=8;
     porta&=0b11110000;//desliga db7...db4
     porta|=0x04;//testa minutos
     delay_us(100);
     if(in_b)res|=4;
     porta&=0b11110000;//desliga db7...db4
     porta|=0x02;//testa sel (seleo)
     delay_us(100);
     if(in_b)res|=2;
     porta&=0b11110000;//desliga db7...db4
     porta|=0x01;//testa stp (stop-pausa-start)
     delay_us(100);
     if(in_b)res|=1;
     enable_interrupts(int_timer0);//religa interrupo
     return res;

}
 //=============================================================================
 //    ROTINA PARA DESLIGAR PISCADAS NO DISPLAY
 //=============================================================================
void stop_pisca(){
  f_piscaA=0;
  f_piscaB=0;
  f_piscaC=0;
  f_piscaD=0;
}

 //=============================================================================
 //    ROTINA DE GERAO DE 3 BIPES AO TERMINO DE UM TEMPO
 //=============================================================================
void bipar(){
   switch (processo){

   case 0:{//um beep
   processo++;
   cont6=TEMPO_BEEP;
   enable_interrupts(int_timer1);//1 bipe
   break;
   }

   case 1: {
       cont6--;
       if(!cont6){
       cont6=TEMPO_PAUSA;
       disable_interrupts(int_timer1);//inicia o intervalo do bipe
       processo++;
       }
       break;
    }

  case 2: {//intervalo do bipe
       cont6--;
       if(!cont6){
       cont6=TEMPO_BEEP;
       processo++;
       enable_interrupts(int_timer1);//2 bipe
       }
       break;
    }

  case 3: {//bipe
       cont6--;
       if(!cont6){
       cont6=TEMPO_PAUSA;
       disable_interrupts(int_timer1);//inicia o intervalo do bipe
       processo++;
       }
       break;
    }


  case 4:{//intervalo do bipe
       cont6--;
       if(!cont6){
       cont6=TEMPO_BEEP;
       processo++;
       enable_interrupts(int_timer1);//3 bipe
       }
       break;
     }

  case 5:{//bipe
       cont6--;
       if(!cont6){
       cont6=TEMPO_BEEP;
       disable_interrupts(int_timer1);//inicia o intervalo do bipe
       processo=0;
       f_bipar=0;//encerra os bipes
       }
       break;
    }
   }//switch processo
}//void bipar()
/*==============================================================================
              ROTINA MAIN
==============================================================================*/
void main() {

   setup_counters(RTCC_INTERNAL,RTCC_DIV_64);
   setup_timer_2(T2_DISABLED,0,1);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   TMR0 = 131; //Carrega TMR0 com o valor
   enable_interrupts(int_timer0);//para a contagem
   enable_interrupts(global);//ativa chave de interrupes
   trisb = 0;//Define PORTb como saidas
   trisa = 0b11110000;//Define PORTa in/out
   portb=0;//reseta portb
   porta=0;//idem
   Lcd_Init(); //Inicializa o display
   env_lcd_c(0x80);//linha 1
   printf(env_lcd_d,"   TetraTimer   ");// nome do aparelho
   env_lcd_c(0xc0);//linha 2
   printf(env_lcd_d,"Larios.Tecnol.ws");//blog
   delay_ms(2000);//tempo de apresentao
   env_lcd_c(1);//limpa tela
   selecao=0xff;
   t1con= 0b00000001;//liga timer 1 internal clock sem prescaller

/*==============================================================================
              Lao Principal de Repetio
==============================================================================*/
   for(;;){
   unsigned int8 temp1;


     if(!le_botao())f_tr1=0;//libera a trava de anti-repetio de tecla apertada
     //=========================================================================
     //   mostra resultados no lcd
     env_lcd_c(0x80);//linha 1
     if(f_piscaA && (timep>2)){ printf(env_lcd_d,"       "); }//apaga
     else{//mostra tempo restante
     printf(env_lcd_d,"A=");//a=
     temp1=Conv_dec(minutosA);
     env_lcd_d('0'| (temp1>>4));//envia dezenas de minutos
     env_lcd_d('0'| (temp1&15));//envia unidade de minutos
     env_lcd_d(':');
     temp1=Conv_dec(segundosA);
     env_lcd_d('0'| (temp1>>4));//envia dezenas de segundos
     env_lcd_d('0'| (temp1&15));//envia unidade de segundos
     }

     env_lcd_c(0x88);//linha 1
      if(f_piscaB && (timep>2)){ printf(env_lcd_d,"       "); }//apaga
     else{//mostra tempo restante
     printf(env_lcd_d,"B=");//b=
     temp1=Conv_dec(minutosB);
     env_lcd_d('0'| (temp1>>4));//envia dezenas de minutos
     env_lcd_d('0'| (temp1&15));//envia unidade de minutos
     env_lcd_d(':');
     temp1=Conv_dec(segundosB);
     env_lcd_d('0'| (temp1>>4));//envia dezenas de segundos
     env_lcd_d('0'| (temp1&15));//envia unidade de segundos
     }

     env_lcd_c(0xC0);//linha 2
      if(f_piscaC && (timep>2)){ printf(env_lcd_d,"       "); }//apaga
     else{//mostra tempo restante
     printf(env_lcd_d,"C=");//c=
     temp1=Conv_dec(minutosC);
     env_lcd_d('0'| (temp1>>4));//envia dezenas de minutos
     env_lcd_d('0'| (temp1&15));//envia unidade de minutos
     env_lcd_d(':');
     temp1=Conv_dec(segundosC);
     env_lcd_d('0'| (temp1>>4));//envia dezenas de segundos
     env_lcd_d('0'| (temp1&15));//envia unidade de segundos
     env_lcd_d(' ');
     }
     env_lcd_c(0xC8);//linha 2
     if(f_piscaD && (timep>2)){ printf(env_lcd_d,"       "); }//apaga
     else{//mostra tempo restante
     printf(env_lcd_d,"D=");//d=
     temp1=Conv_dec(minutosD);
     env_lcd_d('0'| (temp1>>4));//envia dezenas de minutos
     env_lcd_d('0'| (temp1&15));//envia unidade de minutos
     env_lcd_d(':');
     temp1=Conv_dec(segundosD);
     env_lcd_d('0'| (temp1>>4));//envia dezenas de segundos
     env_lcd_d('0'| (temp1&15));//envia unidade de segundos
     }
//===================fim da apresentao no LCD=================================

//==============================testa botes====================================
    if((le_botao()==1)&&!f_tr1 ){//le tecla de START/PAUSE/STOP
        f_tr1=1;//impede repetio setando a trava
        stpisca=0;//reseta contador de piscadas (modo de ajuste)
        switch (selecao) {
        case 0: {if(f_pauseA){f_pauseA=0;releA=0;}else { if(minutosA||segundosA){f_pauseA=1;releA=1;}}break;}
        case 1: {if(f_pauseB){f_pauseB=0;releB=0;}else {if(minutosB||segundosB){f_pauseB=1;releB=1;}}break;}
        case 2: {if(f_pauseC){f_pauseC=0;releC=0;}else {if(minutosC||segundosC){f_pauseC=1;releC=1;}}break;}
        case 3: {if(f_pauseD){f_pauseD=0;releD=0;}else {if(minutosD||segundosD){f_pauseD=1;releD=1;}}break;}
        }
    }
     if((le_botao()==2)&&!f_tr1 ){//le tecla de seleo do canal do temporizador (A...D)
        f_tr1=1;//impede repetio setando a trava
        stpisca=0;//reseta contador de piscadas (modo de ajuste)
        stop_pisca();//desliga piscadas
        selecao++;
        if(selecao>4)selecao=0;
        if(selecao==0)f_piscaA=1;
        if(selecao==1)f_piscaB=1;
        if(selecao==2)f_piscaC=1;
        if(selecao==3)f_piscaD=1;
        }
    if(le_botao()==4){//le tecla de ajuste de minutos
        stpisca=0;//reseta contador de piscadas (modo de ajuste)
        switch (selecao) {
        case 0: {if(f_piscaA&&!releA){minutosA++; if(minutosA>59)minutosA=0;f_pauseA=0;}break;}
        case 1: {if(f_piscaB&&!releB){minutosB++; if(minutosB>59)minutosB=0;f_pauseB=0;}break;}
        case 2: {if(f_piscaC&&!releC){minutosC++; if(minutosC>59)minutosC=0;f_pauseC=0;}break;}
        case 3: {if(f_piscaD&&!releD){minutosD++; if(minutosD>59)minutosD=0;f_pauseD=0;}break;}
       }
    }
      if(le_botao()==8){//le tecla de ajuste de segundos
        stpisca=0;//reseta contador de piscadas (modo de ajuste)
        switch (selecao) {
        case 0: {if(f_piscaA&&!releA){segundosA++; if(segundosA>59)segundosA=0;f_pauseA=0;}break;}
        case 1: {if(f_piscaB&&!releB){segundosB++; if(segundosB>59)segundosB=0;f_pauseB=0;}break;}
        case 2: {if(f_piscaC&&!releC){segundosC++; if(segundosC>59)segundosC=0;f_pauseC=0;}break;}
        case 3: {if(f_piscaD&&!releD){segundosD++; if(segundosD>59)segundosD=0;f_pauseD=0;}break;}
        }
    }

        if((le_botao()==12) &&!f_tr1 ){//le tecla de ajuste de segundos
        stpisca=0;//reseta contador de piscadas (modo de ajuste)
        f_tr1=1;//impede repetio setando a trava
        switch (selecao) {
        case 0: {if(f_piscaA&&!releA){segundosA=0; minutosA=0;f_pauseA=0;releA=0;}break;}
        case 1: {if(f_piscaB&&!releB){segundosB=0; minutosB=0;f_pauseB=0;releB=0;}break;}
        case 2: {if(f_piscaC&&!releC){segundosC=0; minutosC=0;f_pauseC=0;releC=0;}break;}
        case 3: {if(f_piscaD&&!releD){segundosD=0; minutosD=0;f_pauseD=0;releD=0;}break;}
        }
    }
      if(f_bipar)  bipar();
     timep++;//contador usado nas piscadas em modo ajuste
     if(timep>4)timep=0;
     if(stpisca>=TEMPO_PISCANDO){selecao=0xff;stop_pisca();}
     else{ if(!f_tr1)stpisca++; }
    delay_ms(100);//delay (no altere: determina velocidade do ajuste e da piscada do lcd)
   }//for(;;)
}//main()
